/**
  TMR4 Generated Driver File

  @Company
    Microchip Technology Inc.

  @File Name
    tmr4.c

  @Summary
    This is the generated driver implementation file for the TMR4 driver using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  @Description
    This source file provides APIs for TMR4.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.81.6
        Device            :  PIC18F27Q10
        Driver Version    :  2.11
    The generated drivers are tested against the following:
        Compiler          :  XC8 2.30 and above 
        MPLAB 	          :  MPLAB X 5.40
*/

/*
    (c) 2018 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
*/

/**
  Section: Included Files
*/

#include <xc.h>
#include "tmr4.h"

/**
  Section: Global Variables Definitions
 */
extern volatile unsigned int  Timer4_Tick;
extern volatile char Timer4_Flag;
extern volatile unsigned int Max_Timer4_Ticks;


void (*TMR4_InterruptHandler)(void);

/**
  Section: TMR4 APIs
*/
void TMR4_Set_Fosc(uint8_t T4FOSC)
{
     if ( (T4FOSC>0) && (T4FOSC<6)) T4CLKCON=T4FOSC;
}


void TMR4_Initialize(void)
{
    T4CLKCON = 0x04; // T4CS LFINTOSC = 31 Khz 
    // T4CLKCON = 0x01;  /// FOSC/4 =16MHz
    // T4PSYNC Not Synchronized; T4MODE Software control; T4CKPOL Rising Edge; T4CKSYNC Synchronized; 
    T4HLT = 0x20; 
    T4RST = 0x00; // T4RSEL T4INPPS pin;
    T4PR = 0xF1;// PR4 241;
    T4TMR = 0x00; // TMR4 0; 
    PIR4bits.TMR4IF = 0; // Clearing IF flag before enabling the interrupt.
     // Set Default Interrupt Handler
    TMR4_SetInterruptHandler(TMR4_DefaultInterruptHandler);
    //TMR4ON on; T4CKPS =110  Prescaler= 1:64;  T4OUTPS =0001 => postscaler=1:2;
    //T4CON = 0b11100001; //Timer On 0xE1;  divise par 128   DS page 344  1sec
    T4CON = 0b01100001; //Timer OFF 0x41;  divise par 128  DS page 344  1sec
    // T4CON = 0b01111111; // Prescaler=1:128 et postscaler=1:16 =>  2048   8mS   
}

void TMR4_ModeSet(TMR4_HLT_MODE mode)
{
   // Configure different types HLT mode
    T4HLTbits.MODE = mode;
}

void TMR4_ExtResetSourceSet(TMR4_HLT_EXT_RESET_SOURCE reset)
{
    //Configure different types of HLT external reset source
    T4RSTbits.RSEL = reset;
}

void TMR4_Start(void)
{
    // Start the Timer by writing to TMRxON bit
  
    Timer4_Tick=0;
    Timer4_Flag=0;
    PIR4bits.TMR4IF = 0;
    PIE4bits.TMR4IE = 1;
   T4CONbits.TMR4ON = 1;
}

void TMR4_StartTimer(void)
{
    TMR4_Start();
}

void TMR4_Stop(void)
{
    // Stop the Timer by writing to TMRxON bit
    T4CONbits.TMR4ON = 0;
}

void TMR4_StopTimer(void)
{
    TMR4_Stop();
}

uint8_t TMR4_Counter8BitGet(void)
{
    uint8_t readVal;

    readVal = TMR4;

    return readVal;
}

uint8_t TMR4_ReadTimer(void)
{
    return TMR4_Counter8BitGet();
}

void TMR4_Counter8BitSet(uint8_t timerVal)
{
    // Write to the Timer4 register
    TMR4 = timerVal;
}

void TMR4_WriteTimer(uint8_t timerVal)
{
    TMR4_Counter8BitSet(timerVal);
}

void TMR4_Period8BitSet(uint8_t periodVal)
{
   PR4 = periodVal;
}

void TMR4_LoadPeriodRegister(uint8_t periodVal)
{
   TMR4_Period8BitSet(periodVal);
}


void TMR4_ISR(void)
{
    // clear the TMR4 interrupt flag
    PIR4bits.TMR4IF = 0;
 //   Timer4_Tick++;
 // if (Timer4_Tick>10) 
 // {
 //    Timer4_Flag=1;
 //    PIE4bits.TMR4IE = 0;
 //    T4CONbits.TMR4ON = 0;
 //}   
     
    // ticker function call;
    // ticker is 1 -> Callback function gets called everytime this ISR executes
   TMR4_CallBack();
}

void TMR4_CallBack(void)
{
    // Add your custom callback code here
    // this code executes every TMR4_INTERRUPT_TICKER_FACTOR periods of TMR4
    if(TMR4_InterruptHandler)
    {
        TMR4_InterruptHandler();
    }
}

void TMR4_SetInterruptHandler(void (* InterruptHandler)(void)){
    TMR4_InterruptHandler = InterruptHandler;
}

void TMR4_DefaultInterruptHandler(void){
    // add your TMR4 interrupt custom code
    // or set custom function using TMR4_SetInterruptHandler()
        PIR4bits.TMR4IF = 0;
       Timer4_Tick++;
   // 10 secondes delay 
   // avec T4CLKCON = 0x04 =LFINTOSC = 31 Khz
    //if (Timer4_Tick>9)
       // avec FOSC/4=16MHz   diviseur 128x16=2048  PR4=125 =>  16mS
    if (Timer4_Tick>(Max_Timer4_Ticks-1))  // ..2sec
     {
     Timer4_Flag=1;
     PIE4bits.TMR4IE = 0;
     T4CONbits.TMR4ON = 0;
     }   
}

/**
  End of File
*/